import warnings

import pandas as pd

from panorma.exceptions import NotMatchingFields, ParseError


class DataFrame(pd.DataFrame):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        warnings.filterwarnings('ignore')
        self.__class__.__name__ = "DataFrame"
        self._model_fields: dict = self.__class__.__dict__.get("__annotations__")
        self._df_dtypes_dict: dict = self.dtypes.to_dict()
        self._validate_fields_are_matching()
        self._validate_data_types()

    def _validate_fields_are_matching(self):
        model_field_names = [key for key, value in self._model_fields.items()]
        columns = [col for col in self.columns]
        if sorted(model_field_names) != sorted(columns):
            raise NotMatchingFields(model_field_names, columns)

    def _validate_data_types(self):
        for model_field_name, model_field_type in self._model_fields.items():
            try:
                df_field_type_name = self._df_dtypes_dict.get(model_field_name).name.lower()
                if model_field_type.__name__ == "Timestamp":
                    if "datetime" not in df_field_type_name:
                        self[model_field_name] = pd.to_datetime(self[model_field_name])
                    continue
                field_type_name = model_field_type.pd_model.name.lower()
                if field_type_name == df_field_type_name:
                    continue
                self[model_field_name] = self[model_field_name].astype(model_field_type.pd_model())
            except Exception:
                raise ParseError(model_field_name, model_field_type)



class NotMatchingFields(Exception):
    msg = "Fields of the dataframe are not matching the fields of the model"

    def __init__(self, model_field_names: list, columns: list):
        msg = self.msg + f"\n model => {model_field_names} \n columns => {columns}"
        super(NotMatchingFields, self).__init__(msg)


class ParseError(Exception):
    def __init__(self, field_name, field_type):
        msg = f"Impossible to parse {field_name} to {field_type}"
        super(ParseError, self).__init__(msg)



import pandas as pd


class StringDtype(pd.Series):
    pd_model = pd.StringDtype


class Float32Dtype(pd.Series):
    pd_model = pd.Float32Dtype


class Int16Dtype(pd.Series):
    pd_model = pd.Int16Dtype


class CategoricalDtype(pd.Series):
    pd_model = pd.CategoricalDtype


class Timestamp(pd.Series):
    pd_model = pd.Timestamp



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['panorma']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=2.0.1,<3.0.0']

setup_kwargs = {
    'name': 'panorma',
    'version': '0.1.0',
    'description': '',
    'long_description': '# Panorma\nA lightweight Python package (just over 50 lines of code) that enables you to create typed \nmodels for Pandas DataFrames using classes. \n\nYou can easily define  structured models, enforce column typing, \nenjoy autocompletion, and catch invalid column errors early in your DataFrame operations. \n\nSimplify your data modeling and enhance the reliability of your DataFrame workflows.\n\nInstallation:\n- \n```shell script\n  pip install panorma\n```\n\nExample:\n-\n- Create some models:\n```python\nfrom panorma.fields import StringDtype, Int16Dtype, Float32Dtype, Timestamp, CategoricalDtype\nfrom panorma.frames import DataFrame\n\nclass Users(DataFrame):\n    name: StringDtype\n    age: Int16Dtype\n    percentage: Float32Dtype\n    birth_date: Timestamp\n\n\nclass Cars(DataFrame):\n    car: StringDtype\n    mpg: Float32Dtype\n    cylinders: Int16Dtype\n    displacement: Float32Dtype\n    horsepower: Float32Dtype\n    weight: Float32Dtype\n    acceleration: Float32Dtype\n    model: Int16Dtype\n    origin: CategoricalDtype\n```\n\n- Instantiate your models as you instantiate a simple pandas dataframe\n\n```python\nimport pandas as pd\n\nusers = Users({\n    "name": [\'john\', \'kevin\'],\n    "age": [99, 15],\n    "percentage": [0.8, 7.3],\n    "birth_date": [pd.Timestamp(\'20180310\'), pd.Timestamp(\'20230910\')],\n})\ncars = Cars(pd.read_csv(\'CAR_DATASET.csv\'))\n```\n\n- You will get autocompletion for models columns and pandas at the same time\n![image](static/autocompletion.png)\n\n- If the columns of your data are not matching your model, you will get a NotMatchingFields exception:\n![image](static/columns_not_matching.png)\n\n- If a column cannot be cast to the type declared in the model, you will get a ParseError exception:\n![image](static/parse_impossible.png)',
    'author': 'MayasMess',
    'author_email': 'amayas.messara@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
